@@ -67,18 +67,22 @@ class PRF(Enum):
     PRF Setting            PRF    MMD    MUR
     ================= ======== ====== ======
     PRF_19_5_MHZ [*]_ 19.5 MHz  3.1 m  7.7 m
+    PRF_15_6_MHZ      15.6 MHz  5.8 m  9.6 m
     PRF_13_0_MHZ      13.0 MHz  7.0 m 11.5 m
     PRF_8_7_MHZ        8.7 MHz 12.7 m 17.3 m
     PRF_6_5_MHZ        6.5 MHz 18.5 m 23.1 m
+    PRF_5_2_MHZ        5.2 MHz 24.2 m 28.8 m
     ================= ======== ====== ======
 
     .. [*] 19.5MHz is only available for profile 1.
     """
 
     PRF_19_5_MHz = 19500000
+    PRF_15_6_MHz = 15600000
     PRF_13_0_MHz = 13000000
     PRF_8_7_MHz = 8700000
     PRF_6_5_MHz = 6500000
+    PRF_5_2_MHz = 5200000
 
     @property
     def frequency(self) -> int:
