@@ -2,5 +2,5 @@
 # All rights reserved
 
 from .record import H5Record
-from .record_io import load_record, open_record, save_record, save_record_to_h5
+from .record_io import RecordError, load_record, open_record, save_record, save_record_to_h5
 from .recorder import _H5PY_STR_DTYPE, H5Recorder
