@@ -28,11 +28,12 @@ from acconeer.exptool.a121.algo._plugins import (
 from acconeer.exptool.app.new import (
     AppModel,
     Message,
+    PidgetFactoryMapping,
     PluginFamily,
     PluginGeneration,
     PluginPresetBase,
+    pidgets,
 )
-from acconeer.exptool.app.new.ui.plugin_components import PidgetFactoryMapping, pidgets
 
 from ._processor import (
     AmplitudeMethod,
@@ -79,6 +80,10 @@ class ViewPlugin(ProcessorViewPluginBase[ProcessorConfig]):
             "amplitude_method": pidgets.EnumParameterWidgetFactory(
                 enum_type=AmplitudeMethod,
                 name_label_text="Amplitude method:",
+                name_label_tooltip=(
+                    "The method used to calculate the amplitude \
+                    from the complex Sparse IQ data"
+                ),
                 label_mapping={
                     AmplitudeMethod.COHERENT: "Coherent",
                     AmplitudeMethod.NONCOHERENT: "Non-coherent",
