@@ -22,4 +22,16 @@ from .backend import (
 )
 from .pluginbase import PlotPluginBase, PluginPresetBase, PluginSpecBase, ViewPluginBase
 from .storage import get_temp_dir, get_temp_h5_path
-from .ui import BUTTON_ICON_COLOR
+from .ui import (
+    BUTTON_ICON_COLOR,
+    AttrsConfigEditor,
+    GridGroupBox,
+    HorizontalGroupBox,
+    PidgetFactoryMapping,
+    SessionConfigEditor,
+    SmartMetadataView,
+    SmartPerfCalcView,
+    TwoSensorIdsEditor,
+    VerticalGroupBox,
+    pidgets,
+)
