@@ -12,6 +12,7 @@ from pathlib import Path
 from typing import Any, Callable, List, Optional, Tuple
 from uuid import UUID
 
+import attrs
 from typing_extensions import Protocol
 
 from PySide6.QtCore import QDeadlineTimer, QObject, QThread, Signal
@@ -41,7 +42,7 @@ from acconeer.exptool.app.new.backend import (
     StatusMessage,
 )
 from acconeer.exptool.app.new.storage import remove_temp_dir
-from acconeer.exptool.utils import USBDevice  # type: ignore[import]
+from acconeer.exptool.utils import CommDevice, SerialDevice, USBDevice  # type: ignore[import]
 
 from .plugin_protocols import PlotPluginInterface, ViewPluginInterface
 from .port_updater import PortUpdater
@@ -113,6 +114,17 @@ class _BackendListeningThread(QThread):
         log.debug("Backend listening thread stopping...")
 
 
+@attrs.mutable(kw_only=True)
+class _Config:
+    connection_interface: ConnectionInterface = ConnectionInterface.SERIAL
+    socket_connection_ip: str = ""
+    serial_connection_device: Optional[SerialDevice] = None
+    usb_connection_device: Optional[USBDevice] = None
+    overridden_baudrate: Optional[int] = None
+    autoconnect_enabled: bool = False
+    recording_enabled: bool = True
+
+
 class AppModel(QObject):
     sig_notify = Signal(object)
     sig_error = Signal(Exception, object)
@@ -122,6 +134,7 @@ class AppModel(QObject):
     sig_status_message = Signal(object)
     sig_rate_stats = Signal(float, bool, float, bool)
     sig_backend_cpu_percent = Signal(int)
+    sig_frame_count = Signal(object)
     sig_backend_state_changed = Signal(object)
 
     plugins: list[PluginSpec]
@@ -131,18 +144,11 @@ class AppModel(QObject):
 
     connection_state: ConnectionState
     connection_warning: Optional[str]
-    connection_interface: ConnectionInterface
     plugin_state: PluginState
-    socket_connection_ip: str
-    serial_connection_port: Optional[str]
-    available_tagged_ports: list[Tuple[str, Optional[str]]]
-    usb_connection_device: Optional[USBDevice]
-    available_usb_devices: list[USBDevice]
-    overridden_baudrate: Optional[int]
+    available_serial_devices: List[SerialDevice]
+    available_usb_devices: List[USBDevice]
 
     saveable_file: Optional[Path]
-    autoconnect_enabled: bool
-    recording_enabled: bool
 
     def __init__(self, backend: Backend, plugins: list[PluginSpec]) -> None:
         super().__init__()
@@ -157,6 +163,8 @@ class AppModel(QObject):
 
         self._a121_server_info: Optional[a121.ServerInfo] = None
 
+        self._config = _Config()
+
         self.plugins = plugins
         self.plugin = None
 
@@ -164,19 +172,12 @@ class AppModel(QObject):
 
         self.connection_state = ConnectionState.DISCONNECTED
         self.connection_warning = None
-        self.connection_interface = ConnectionInterface.SERIAL
         self.plugin_state = PluginState.UNLOADED
-        self.socket_connection_ip = ""
-        self.serial_connection_port = None
-        self.usb_connection_device = None
-        self.available_tagged_ports = []
+
+        self.available_serial_devices = []
         self.available_usb_devices = []
-        self.overridden_baudrate = None
 
         self.saveable_file = None
-        self.autoconnect_enabled = False
-
-        self.recording_enabled = True
 
     def start(self) -> None:
         self._listener.start()
@@ -190,14 +191,14 @@ class AppModel(QObject):
         if self.connection_state in [ConnectionState.CONNECTING, ConnectionState.CONNECTED]:
             self.disconnect_client()
 
-        wait_start_time = time.time()
+        wait_start_time = time.monotonic()
         while (
             self.plugin_state != PluginState.UNLOADED
             and self.connection_state != ConnectionState.DISCONNECTED
         ):  # TODO: Do this better
             QApplication.processEvents()
 
-            if (time.time() - wait_start_time) > WAIT_FOR_UNLOAD_TIMEOUT:
+            if (time.monotonic() - wait_start_time) > WAIT_FOR_UNLOAD_TIMEOUT:
                 log.error("Plugin not unloaded on stop")
                 break
 
@@ -338,6 +339,8 @@ class AppModel(QObject):
             )
         elif message.name == "cpu_percent":
             self.sig_backend_cpu_percent.emit(message.data)
+        elif message.name == "frame_count":
+            self.sig_frame_count.emit(message.data)
         else:
             raise RuntimeError(f"Got unknown general message '{message.name}'")
 
@@ -348,10 +351,9 @@ class AppModel(QObject):
         self.saveable_file = path
         self.broadcast()
 
-    def _is_serial_device_unflashed(self, serial_port: Optional[str]) -> bool:
-        for port, tag in self.available_tagged_ports:
-            if port == serial_port and tag and "unflashed" in tag.lower():
-                return True
+    def _is_serial_device_unflashed(self, serial_device: Optional[SerialDevice]) -> bool:
+        if serial_device and serial_device.unflashed:
+            return True
         return False
 
     def _is_usb_device_unflashed(self, usb_device: Optional[USBDevice]) -> bool:
@@ -366,69 +368,70 @@ class AppModel(QObject):
 
     def _handle_port_update(
         self,
-        tagged_ports: list[Tuple[str, Optional[str]]],
-        usb_devices: Optional[list[USBDevice]],
+        serial_devices: list[SerialDevice],
+        usb_devices: list[USBDevice],
     ) -> None:
-        tagged_ports_map = dict(tagged_ports)
         if self.connection_state is not ConnectionState.DISCONNECTED and (
             (
-                self.connection_interface == ConnectionInterface.SERIAL
-                and self.serial_connection_port not in tagged_ports_map.keys()
+                self._config.connection_interface == ConnectionInterface.SERIAL
+                and self._config.serial_connection_device not in serial_devices
             )
             or (
-                self.connection_interface == ConnectionInterface.USB
-                and usb_devices
-                and self.usb_connection_device not in usb_devices
+                self._config.connection_interface == ConnectionInterface.USB
+                and self._config.usb_connection_device not in usb_devices
             )
         ):
             self.disconnect_client()
-        self.serial_connection_port, recognized = self._select_new_serial_port(
-            dict(self.available_tagged_ports),
-            tagged_ports_map,
-            self.serial_connection_port,
+        self._config.serial_connection_device, recognized = self._select_new_device(
+            self.available_serial_devices,
+            serial_devices,
+            self._config.serial_connection_device,
         )
 
-        self.available_tagged_ports = tagged_ports
+        self.available_serial_devices = serial_devices
         connect = False
 
         if recognized:
             self.set_connection_interface(ConnectionInterface.SERIAL)
-            if self._is_serial_device_unflashed(self.serial_connection_port):
+            if self._is_serial_device_unflashed(self._config.serial_connection_device):
                 connect = False
                 self.send_warning_message(
-                    f"Found unflashed device at serial port: {self.serial_connection_port}"
+                    "Found unflashed device at serial port:"
+                    f" {self._config.serial_connection_device}"
                 )
             else:
                 connect = True
-                self.send_status_message(f"Recognized serial port: {self.serial_connection_port}")
+                self.send_status_message(
+                    f"Recognized serial port: {self._config.serial_connection_device}"
+                )
 
         if usb_devices is not None:
-            self.usb_connection_device, recognized = self._select_new_usb_device(
-                usb_devices, self.usb_connection_device
+            self._config.usb_connection_device, recognized = self._select_new_device(
+                self.available_usb_devices, usb_devices, self._config.usb_connection_device
             )
 
             self.available_usb_devices = usb_devices
 
             if recognized:
-                assert self.usb_connection_device is not None
+                assert self._config.usb_connection_device is not None
                 self.set_connection_interface(ConnectionInterface.USB)
-                if self._is_usb_device_unflashed(self.usb_connection_device):
+                if self._is_usb_device_unflashed(self._config.usb_connection_device):
                     connect = False
                     self.send_warning_message(
-                        f"Found unflashed USB device: {self.usb_connection_device}"
+                        f"Found unflashed USB device: {self._config.usb_connection_device}"
                     )
-                elif self._is_usb_device_inaccessible(self.usb_connection_device):
+                elif self._is_usb_device_inaccessible(self._config.usb_connection_device):
                     connect = False
                     self.send_warning_message(
-                        f"Found inaccessible USB device: {self.usb_connection_device}"
+                        f"Found inaccessible USB device: {self._config.usb_connection_device}"
                     )
                 else:
                     connect = True
                     self.send_status_message(
-                        f"Recognized USB device: {self.usb_connection_device}"
+                        f"Recognized USB device: {self._config.usb_connection_device}"
                     )
 
-        if connect and self.autoconnect_enabled:
+        if connect and self._config.autoconnect_enabled:
             self._autoconnect()
 
         self.broadcast()
@@ -436,61 +439,41 @@ class AppModel(QObject):
     def _autoconnect(self) -> None:
         self.connect_client(auto=True)
 
-    def _select_new_serial_port(
+    def _select_new_device(
         self,
-        old_ports: dict[str, Optional[str]],
-        new_ports: dict[str, Optional[str]],
-        current_port: Optional[str],
-    ) -> Tuple[Optional[str], bool]:
+        old_devices: list[CommDevice],
+        new_devices: list[CommDevice],
+        current_port: Optional[CommDevice],
+    ) -> Tuple[Optional[CommDevice], bool]:
         if self.connection_state != ConnectionState.DISCONNECTED:
             return current_port, False
 
-        if current_port not in new_ports:  # Then find a new suitable port
-            port = None
-
-            for port, tag in new_ports.items():
-                if tag:
-                    return port, (current_port is None)
-
-            return port, False
-
-        # If we already have a tagged port, keep it
-        if new_ports[current_port]:
-            return current_port, False
-
-        # If a tagged port was added, select it
-        added_ports = {k: v for k, v in new_ports.items() if k not in old_ports}
-        for port, tag in added_ports.items():
-            if tag:
-                return port, True
-
-        return current_port, False
-
-    def _select_new_usb_device(
-        self,
-        new_ports: list[USBDevice],
-        current_port: Optional[USBDevice],
-    ) -> Tuple[Optional[USBDevice], bool]:
-        if self.connection_state != ConnectionState.DISCONNECTED:
-            return current_port, False
-
-        if not new_ports:
+        if not new_devices:
             return None, False
 
-        if current_port not in new_ports:
-            return new_ports[0], True
+        added_devices = [dev for dev in new_devices if dev not in old_devices]
+        for device in added_devices:
+            if device.recognized:
+                return device, True
+
+        if current_port not in new_devices:
+            return new_devices[0], True
 
         return current_port, False
 
     def connect_client(self, auto: bool = False) -> None:
-        if self.connection_interface == ConnectionInterface.SOCKET:
-            client_info = a121.ClientInfo(ip_address=self.socket_connection_ip)
-        elif self.connection_interface == ConnectionInterface.SERIAL:
+        if self._config.connection_interface == ConnectionInterface.SOCKET:
+            client_info = a121.ClientInfo(ip_address=self._config.socket_connection_ip)
+        elif (
+            self._config.connection_interface == ConnectionInterface.SERIAL
+            and self._config.serial_connection_device is not None
+        ):
             client_info = a121.ClientInfo(
-                serial_port=self.serial_connection_port, override_baudrate=self.overridden_baudrate
+                serial_port=self._config.serial_connection_device.port,
+                override_baudrate=self._config.overridden_baudrate,
             )
-        elif self.connection_interface == ConnectionInterface.USB:
-            client_info = a121.ClientInfo(usb_device=self.usb_connection_device)
+        elif self._config.connection_interface == ConnectionInterface.USB:
+            client_info = a121.ClientInfo(usb_device=self._config.usb_connection_device)
         else:
             raise RuntimeError
 
@@ -518,17 +501,32 @@ class AppModel(QObject):
 
     def is_connect_ready(self) -> bool:
         return (
-            (self.connection_interface == ConnectionInterface.SOCKET)
+            (self._config.connection_interface == ConnectionInterface.SOCKET)
             or (
-                self.connection_interface == ConnectionInterface.SERIAL
-                and self.serial_connection_port is not None
-                and not self._is_serial_device_unflashed(self.serial_connection_port)
+                self._config.connection_interface == ConnectionInterface.SERIAL
+                and self._config.serial_connection_device is not None
+                and not self._is_serial_device_unflashed(self._config.serial_connection_device)
             )
             or (
-                self.connection_interface == ConnectionInterface.USB
-                and self.usb_connection_device is not None
-                and not self._is_usb_device_unflashed(self.usb_connection_device)
-                and not self._is_usb_device_inaccessible(self.usb_connection_device)
+                self._config.connection_interface == ConnectionInterface.USB
+                and self._config.usb_connection_device is not None
+                and not self._is_usb_device_unflashed(self._config.usb_connection_device)
+                and not self._is_usb_device_inaccessible(self._config.usb_connection_device)
+            )
+        )
+
+    def is_ready_for_session(self) -> bool:
+        """
+        Returns True if the plugin is ready for a new session.
+        Additional conditions can be added in respective plugin.
+        """
+        return (
+            self.plugin_state == PluginState.LOADED_IDLE
+            and self.connection_state == ConnectionState.CONNECTED
+            and self._a121_server_info is not None
+            and any(
+                sensor_info.connected
+                for _, sensor_info in self._a121_server_info.sensor_infos.items()
             )
         )
 
@@ -543,20 +541,48 @@ class AppModel(QObject):
         else:
             self._port_updater.resume()
 
+    @property
+    def connection_interface(self) -> ConnectionInterface:
+        return self._config.connection_interface
+
+    @property
+    def socket_connection_ip(self) -> str:
+        return self._config.socket_connection_ip
+
+    @property
+    def serial_connection_device(self) -> Optional[SerialDevice]:
+        return self._config.serial_connection_device
+
+    @property
+    def usb_connection_device(self) -> Optional[USBDevice]:
+        return self._config.usb_connection_device
+
+    @property
+    def autoconnect_enabled(self) -> bool:
+        return self._config.autoconnect_enabled
+
+    @property
+    def overridden_baudrate(self) -> Optional[int]:
+        return self._config.overridden_baudrate
+
+    @property
+    def recording_enabled(self) -> bool:
+        return self._config.recording_enabled
+
     def set_connection_interface(self, connection_interface: ConnectionInterface) -> None:
-        self.connection_interface = connection_interface
+        self._config.connection_interface = connection_interface
         self.broadcast()
 
     def set_socket_connection_ip(self, ip: str) -> None:
-        self.socket_connection_ip = ip
+        self._config.socket_connection_ip = ip
         self.broadcast()
 
-    def set_serial_connection_port(self, port: Optional[str]) -> None:
-        self.serial_connection_port = port
+    def set_serial_connection_device(self, device: Optional[SerialDevice]) -> None:
+        self._config.serial_connection_device = device
         self.broadcast()
 
-    def set_usb_connection_port(self, port: Optional[str]) -> None:
-        self.usb_connection_device = port
+    def set_usb_connection_device(self, device: Optional[USBDevice]) -> None:
+        self._config.usb_connection_device = device
         self.broadcast()
 
     def set_plugin_state(self, state: PluginState) -> None:
@@ -564,15 +590,15 @@ class AppModel(QObject):
         self.broadcast()
 
     def set_autoconnect_enabled(self, autoconnect_enabled: bool) -> None:
-        self.autoconnect_enabled = autoconnect_enabled
+        self._config.autoconnect_enabled = autoconnect_enabled
         self.broadcast()
 
     def set_overridden_baudrate(self, overridden_baudrate: Optional[int]) -> None:
-        self.overridden_baudrate = overridden_baudrate
+        self._config.overridden_baudrate = overridden_baudrate
         self.broadcast()
 
     def set_recording_enabled(self, recording_enabled: bool) -> None:
-        self.recording_enabled = recording_enabled
+        self._config.recording_enabled = recording_enabled
         self.broadcast()
 
     def _unload_current_plugin(self) -> None:
