@@ -12,7 +12,6 @@ from tempfile import TemporaryDirectory
 from typing import Optional, Tuple
 
 import qtawesome as qta
-import serial
 from requests import Response, Session
 from requests.cookies import RequestsCookieJar
 
@@ -36,6 +35,7 @@ from PySide6.QtWidgets import (
 
 from acconeer.exptool.app import resources  # type: ignore[attr-defined]
 from acconeer.exptool.app.new._enums import ConnectionInterface, ConnectionState
+from acconeer.exptool.app.new._exceptions import HandledException
 from acconeer.exptool.app.new.app_model import AppModel
 from acconeer.exptool.app.new.ui.misc import ExceptionWidget
 from acconeer.exptool.flash import (  # type: ignore[import]
@@ -44,14 +44,15 @@ from acconeer.exptool.flash import (  # type: ignore[import]
     DevLicense,
     clear_cookies,
     download,
-    find_flash_port,
     flash_image,
     get_content,
     get_cookies,
+    get_flash_download_name,
+    get_flash_known_devices,
     login,
     save_cookies,
 )
-from acconeer.exptool.utils import USBDevice  # type: ignore[import]
+from acconeer.exptool.utils import CommDevice  # type: ignore[import]
 
 from .misc import BUTTON_ICON_COLOR, SerialPortComboBox, USBDeviceComboBox
 
@@ -68,19 +69,34 @@ log = logging.getLogger(__name__)
 class _FlashThread(QThread):
     flash_failed = Signal(Exception, str)
     flash_done = Signal()
+    flash_progress = Signal(int)
 
-    def __init__(self, bin_file: str, flash_port: serial.tools.list_ports.ListPortInfo) -> None:
+    def __init__(
+        self, bin_file: str, flash_device: CommDevice, device_name: Optional[str]
+    ) -> None:
         super().__init__()
         self.bin_file = bin_file
-        self.flash_port = flash_port
+        self.flash_device = flash_device
+        self.device_name = device_name
 
     def run(self) -> None:
+        def progress_callback(progress: int, end: bool = False) -> None:
+            if end:
+                self.flash_progress.emit(100)
+            else:
+                self.flash_progress.emit(progress)
+
         try:
-            flash_image(self.bin_file, self.flash_port)
+            flash_image(
+                self.bin_file,
+                self.flash_device,
+                device_name=self.device_name,
+                progress_callback=progress_callback,
+            )
             self.flash_done.emit()
         except Exception as e:
             log.error(str(e))
-            self.flash_failed.emit(e, traceback.format_exc())
+            self.flash_failed.emit(HandledException(e), traceback.format_exc())
 
 
 class _FlashDialog(QDialog):
@@ -111,15 +127,21 @@ class _FlashDialog(QDialog):
         self.flash_label.setAlignment(Qt.AlignCenter)
         vbox.addWidget(self.flash_label)
 
+        self.ok_button = QPushButton(self)
+        self.ok_button.setText("OK")
+        self.ok_button.setHidden(True)
+        self.ok_button.clicked.connect(self._on_ok_click)
+        vbox.addWidget(self.ok_button)
         self.setLayout(vbox)
 
-    def flash(self, bin_file: str, flash_port: serial.tools.list_ports.ListPortInfo) -> None:
-        self.flash_thread = _FlashThread(bin_file, flash_port)
+    def flash(self, bin_file: str, flash_device: CommDevice, device_name: Optional[str]) -> None:
+        self.flash_thread = _FlashThread(bin_file, flash_device, device_name)
         self.flash_thread.started.connect(self._flash_start)
         self.flash_thread.finished.connect(self.flash_thread.deleteLater)
         self.flash_thread.finished.connect(self._flash_stop)
         self.flash_thread.flash_done.connect(self._flash_done)
         self.flash_thread.flash_failed.connect(self._flash_failed)
+        self.flash_thread.flash_progress.connect(self._flash_progress)
 
         self.flash_thread.start()
         self._flashing = True
@@ -128,6 +150,7 @@ class _FlashDialog(QDialog):
     def _flash_start(self) -> None:
         self.flash_label.setText("Flashing...")
         self.loading.show()
+        self.ok_button.setHidden(True)
         self.flash_movie.start()
 
     def _flash_stop(self) -> None:
@@ -135,14 +158,23 @@ class _FlashDialog(QDialog):
         self.loading.hide()
         self._flashing = False
 
+    def _flash_progress(self, progress: int) -> None:
+        if self._flashing:
+            self.flash_label.setText(f"Flashing... {progress}%")
+
     def _flash_done(self) -> None:
         self.flash_done.emit()
         self.flash_label.setText("Flashing done!")
+        self.ok_button.setHidden(False)
 
     def _flash_failed(self, exception: Exception, traceback_str: Optional[str]) -> None:
         self.flash_done.emit()
         self.flash_label.setText("Flashing failed!")
         ExceptionWidget(self, exc=exception, traceback_str=traceback_str).exec()
+        self.ok_button.setHidden(False)
+
+    def _on_ok_click(self) -> None:
+        self.close()
 
     def closeEvent(self, event: QCloseEvent) -> None:
         if self._flashing:
@@ -200,6 +232,11 @@ class _FlashPopup(QDialog):
 
         self.interface_dd.currentIndexChanged.connect(self._on_interface_dd_change)
 
+        self.device_selection = QComboBox(self)
+        self.device_selection.setEnabled(False)
+        self.device_name_selection = None
+        self.device_selection.currentIndexChanged.connect(self._on_device_selection_change)
+
         self.stacked = QStackedWidget(self)
         self.stacked.setStyleSheet("QStackedWidget {background-color: transparent;}")
         self.stacked.addWidget(SerialPortComboBox(app_model, self.stacked))
@@ -224,8 +261,9 @@ class _FlashPopup(QDialog):
         layout.addWidget(self.file_label,               0,   3,   1,     9)    # noqa: E241
         layout.addWidget(self.get_latest_button,        1,   0,   1,     3)    # noqa: E241
         layout.addWidget(self.downloaded_version_label, 1,   3,   1,     9)    # noqa: E241
-        layout.addWidget(self.interface_dd,             2,   0,   1,     6)    # noqa: E241
-        layout.addWidget(self.stacked,                  2,   6,   1,     6)    # noqa: E241
+        layout.addWidget(self.interface_dd,             2,   0,   1,     3)    # noqa: E241
+        layout.addWidget(self.stacked,                  2,   3,   1,     6)    # noqa: E241
+        layout.addWidget(self.device_selection,         2,   9,   1,     3)    # noqa: E241
         layout.addWidget(self.flash_button,             3,   0,   1,    12)    # noqa: E241
         layout.addWidget(self.logged_in_label,          4,   0,   1,     8)    # noqa: E241
         layout.addWidget(self.logout_button,            4,   8,   1,     4)    # noqa: E241
@@ -244,13 +282,20 @@ class _FlashPopup(QDialog):
         app_model.sig_notify.connect(self._on_app_model_update)
 
     @property
-    def flash_port(self) -> Optional[str]:
+    def flash_device(self) -> Optional[CommDevice]:
         if self.app_model.connection_interface == ConnectionInterface.SERIAL:
-            return self.app_model.serial_connection_port
+            return self.app_model.serial_connection_device
         if self.app_model.connection_interface == ConnectionInterface.USB:
             return self.app_model.usb_connection_device
         return None
 
+    @property
+    def device_name(self) -> Optional[str]:
+        if not self.device_selection.isEnabled():
+            return None
+        else:
+            return self.device_name_selection
+
     def _get_latest_bin_file(self) -> None:
         self.auth_thread = _AuthThread(dev_license=self.dev_license)
         self.auth_thread.started.connect(self._auth_start)
@@ -297,10 +342,11 @@ class _FlashPopup(QDialog):
 
         license_agreement_dialog = LicenseAgreementDialog(self.dev_license, self)
         license_accepted = license_agreement_dialog.exec()
+        device_download_name = get_flash_download_name(self.flash_device, self.device_name)
 
         if license_accepted:
             self.download_thread = BinDownloadThread(
-                self._get_device_name(), cookies, content[1], content[2]
+                device_download_name, cookies, content[1], content[2]
             )
 
             self.download_thread.started.connect(self._download_start)
@@ -350,14 +396,10 @@ class _FlashPopup(QDialog):
         self.adjustSize()
 
     def _flash(self) -> None:
-        flash_port = find_flash_port(
-            self.flash_port, do_log=False, use_serial=not isinstance(self.flash_port, USBDevice)
-        )
-
         assert self.bin_file is not None
 
         self.app_model.set_port_updates_pause(True)
-        self.flash_dialog.flash(self.bin_file, flash_port)
+        self.flash_dialog.flash(self.bin_file, self.flash_device, self.device_name)
 
     def _flash_done(self) -> None:
         self.app_model.set_port_updates_pause(False)
@@ -365,6 +407,9 @@ class _FlashPopup(QDialog):
     def _on_interface_dd_change(self) -> None:
         self.app_model.set_connection_interface(self.interface_dd.currentData())
 
+    def _on_device_selection_change(self) -> None:
+        self.device_name_selection = self.device_selection.currentText()
+
     def _on_app_model_update(self, app_model: AppModel) -> None:
         if app_model.connection_interface in [ConnectionInterface.SERIAL, ConnectionInterface.USB]:
             interface_index = self.interface_dd.findData(app_model.connection_interface)
@@ -374,19 +419,37 @@ class _FlashPopup(QDialog):
             self.interface_dd.setCurrentIndex(interface_index)
             self.stacked.setCurrentIndex(interface_index)
 
-            enable_select = (
-                find_flash_port(do_log=False) is not None
-                and not self.authenticating
-                and not self.downloading_firmware
-            )
+            current_index = self.device_selection.currentIndex()
+            if self.flash_device is None or (
+                self.flash_device is not None and self.flash_device.name is not None
+            ):
+                self.device_selection.clear()
+                self.device_selection.setEnabled(False)
+            else:
+                self.device_selection.clear()
+                for dev in get_flash_known_devices():
+                    self.device_selection.addItem(dev)
+                self.device_selection.setEnabled(True)
+                if current_index >= 0:
+                    self.device_selection.setCurrentIndex(current_index)
+
+            enable_select = not self.authenticating and not self.downloading_firmware
             self.get_latest_button.setEnabled(enable_select)
+        else:
+            self.device_selection.clear()
+            self.device_selection.setEnabled(False)
 
         self._draw()
 
     def _draw(self) -> None:
         self.file_label.setText(self.bin_file if self.bin_file else "")
         self.file_label.setEnabled(self.bin_file is not None)
-        self.flash_button.setEnabled(self.flash_port is not None and self.bin_file is not None)
+        self.flash_button.setEnabled(
+            not self.authenticating
+            and not self.downloading_firmware
+            and self.flash_device is not None
+            and self.bin_file is not None
+        )
 
     def exec(self) -> None:
         if self.app_model.connection_interface == ConnectionInterface.SOCKET:
@@ -394,17 +457,6 @@ class _FlashPopup(QDialog):
 
         super().exec()
 
-    def _get_device_name(self) -> str | None:
-        flash_port = find_flash_port()
-
-        if flash_port is not None:
-            if isinstance(flash_port, USBDevice):
-                return str(flash_port.name)
-            else:
-                return str(flash_port.product)
-
-        return None
-
     def closeEvent(self, event: QCloseEvent) -> None:
         if self.authenticating:
             self.auth_thread.terminate()
@@ -447,6 +499,7 @@ class FlashButton(QPushButton):
         self.setFixedWidth(100)
         self.setText("Flash")
         self.setIcon(qta.icon("mdi.flash", color=BUTTON_ICON_COLOR))
+        self.setToolTip("Flash the device with a bin file")
 
         app_model.sig_notify.connect(self._on_app_model_update)
         self.pop_up = _FlashPopup(app_model, self)
